'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _createReactClass = require('create-react-class');

var _createReactClass2 = _interopRequireDefault(_createReactClass);

var _Option = require('./Option');

var _Option2 = _interopRequireDefault(_Option);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _InertSelect = require('./InertSelect');

var _InertSelect2 = _interopRequireDefault(_InertSelect);

var _selectBackingHandlers = require('./selectBackingHandlers');

var _selectBackingHandlers2 = _interopRequireDefault(_selectBackingHandlers);

var _selectInertHandlers = require('./selectInertHandlers');

var _selectInertHandlers2 = _interopRequireDefault(_selectInertHandlers);

var _utilityFunctions = require('./utilityFunctions');

var _utilityFunctions2 = _interopRequireDefault(_utilityFunctions);

var ESCAPE_KEY = 27;
var TAB_KEY = 9;
var hiddenSelectStyle = {
  // HACK: for Safari. If the hidden select has
  // size 0 then it is skipped in the taborder.
  height: 0.1,
  width: 0.1,
  margin: 0,
  border: 0,
  padding: 0,
  outline: 'none',
  opacity: 0,
  position: 'absolute'
};

var Select = (0, _createReactClass2['default'])(_extends({
  propTypes: {
    value: _propTypes2['default'].oneOfType([_propTypes2['default'].string, _propTypes2['default'].number]).isRequired,
    children: _propTypes2['default'].arrayOf(_propTypes2['default'].node).isRequired,
    autoComplete: _propTypes2['default'].string,
    containerClassName: _propTypes2['default'].string,
    optionsContainerClassName: _propTypes2['default'].string,

    // The name is an unimportant string we attach to the
    // <select> for purposes of testing.
    name: _propTypes2['default'].string,

    disableDropdown: _propTypes2['default'].bool,
    useNative: _propTypes2['default'].bool,

    onOptionHover: _propTypes2['default'].func.isRequired,
    onOptionActive: _propTypes2['default'].func.isRequired,

    onChange: _propTypes2['default'].func.isRequired,
    onFocus: _propTypes2['default'].func,
    onBlur: _propTypes2['default'].func,

    style: _propTypes2['default'].shape({
      containerStyle: _propTypes2['default'].object,
      selectContainerStyle: _propTypes2['default'].object,
      optionsContainerStyle: _propTypes2['default'].object
    }),
    displayingChildRenderer: _propTypes2['default'].func
  },

  getDefaultProps: function getDefaultProps() {
    return {
      onOptionHover: function onOptionHover() {},
      onOptionActive: function onOptionActive() {},
      onChange: function onChange() {},
      style: {}
    };
  },

  getInitialState: function getInitialState() {
    return {
      isExpanded: false,
      isFocused: false,

      // This sequence of events occurs when we click on
      // an option in the inert select:
      // mouseDown + backing select blur -> mouseup
      // we keep
      isInVirtualFocusState: false,

      hoverIndex: undefined,
      activeIndex: undefined
    };
  },

  /*
    Global Blur Event Listeners
   */
  componentDidMount: function componentDidMount() {
    var _this = this;

    if (this.props.useNative) {
      return;
    }

    try {
      this.globalEventListener = window.addEventListener('click', function (e) {
        var inertSelect = _reactDom2['default'].findDOMNode(_this._inertSelect);

        if (!inertSelect) {
          return;
        }

        if (inertSelect.contains(e.target)) {
          return;
        }

        _this.setState({
          isFocused: false,
          isExpanded: false,
          activeIndex: undefined,
          hoverIndex: undefined
        });

        // On Chrome, the click event does not appear to get transmitted,
        // causing the focus to still stay on the backing select. This
        // makes future keystrokes get transmitted to the backing select.
        if (e.target.focus) {
          e.target.focus();
        }
      });

      // An escape keypress closes the dropdown but does not make the
      // select lose focus.
      this.keyupEventListener = window.addEventListener('keyup', function (e) {
        if (e.keyCode === ESCAPE_KEY) {
          _this.setState({
            isExpanded: false,
            activeIndex: undefined,
            hoverIndex: undefined
          });
        }
      });

      // A tab keypress while the menu is expanded does nothing at all. The
      // focus and selection remain unchanged.
      // Note that this is the behavior in Chrome and Safari. In Firefox a tab
      // while the menu is focused immediately moves focus to then next tabindex.
      // Firefox selects, however, behave quite differently in general. Elements
      // are selected immediately through keyboard navigation, for one, so it makes
      // sense for a tab to just change focus.
      this.keydownEventListener = window.addEventListener('keydown', function (e) {
        if (e.keyCode === TAB_KEY && _this.state.isExpanded) {
          e.preventDefault();
          e.stopPropagation();
          _this._backingSelect.focus();
          _this.setState({
            isFocused: true
          });
        }
      });
    } catch (e) {}
  },

  componentWillUnmount: function componentWillUnmount() {
    try {
      window.removeEventListener(this.globalEventListener);
    } catch (e) {}
    try {
      window.removeEventListener(this.keyupEventListener);
    } catch (e) {}
    try {
      window.removeEventListener(this.keydownEventListener);
    } catch (e) {}
  }

}, _selectBackingHandlers2['default'], _selectInertHandlers2['default'], _utilityFunctions2['default'], {

  buildBackingSelect: function buildBackingSelect() {
    var _this2 = this;

    var children = _react2['default'].Children.toArray(this.props.children);
    return _react2['default'].createElement(
      'select',
      _extends({}, this.generateBackingHandlers(), {
        style: this.props.useNative ? {} : hiddenSelectStyle,
        ref: function (s) {
          return _this2._backingSelect = s;
        },
        // HACK: for Firefox!
        // Further reading: this most fun issue https://bugzilla.mozilla.org/show_bug.cgi?id=126379
        size: this.props.useNative ? '1' : '2',
        autoComplete: this.props.autoComplete,
        name: this.props.name,
        value: this.props.value
      }),
      children.map(function (c, i) {
        (0, _invariant2['default'])(c.type === _Option2['default'], 'The Select component should\n          only take Select.Option instances as children.');
        return _react2['default'].createElement(
          'option',
          { value: c.props.value, key: c.props.value },
          c.props.label || c.props.value
        );
      })
    );
  },

  renderInertSelect: function renderInertSelect() {
    var _this3 = this;

    return _react2['default'].createElement(
      _InertSelect2['default'],
      _extends({
        ref: function (s) {
          return _this3._inertSelect = s;
        }
      }, this.generateInertHandlers(), {
        value: this.props.value,
        isExpanded: this.state.isExpanded,
        isFocused: this.state.isFocused,
        disableDropdown: this.props.disableDropdown,
        containerClassName: this.props.containerClassName,
        optionsContainerClassName: this.props.optionsContainerClassName,
        displayingChildRenderer: this.props.displayingChildRenderer,
        style: this.props.style,
        hoverIndex: this.state.hoverIndex,
        activeIndex: this.state.activeIndex,
        selectedIndex: this.getSelectedIndex(this.props.value)
      }),
      this.props.children
    );
  },

  render: function render() {
    return _react2['default'].createElement(
      'div',
      { style: this.props.style.containerStyle || {} },
      this.buildBackingSelect(),
      !this.props.useNative && this.renderInertSelect()
    );
  }
}));

exports['default'] = Select;
module.exports = exports['default'];
/*
  Imports methods from three other files so that
  this one is not ridiculously sized.
 */