"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = {
  /*
   INERT HANDLERS
   These handlers respond to events on the inert select
   and help to make it behave like a real select, including
   opening it, closing it, hovering over elements, keyboard
   control selections, and so forth.
   */
  generateInertHandlers: function generateInertHandlers() {
    return {
      onExpanded: this.handleInertSelectExpanded,
      onClosed: this.handleInertSelectClosed,

      onHoverIndex: this.handleInertHoverIndex,
      onActiveIndex: this.handleInertActiveIndex,
      onSelectIndex: this.handleInertSelectIndex
    };
  },

  handleInertSelectExpanded: function handleInertSelectExpanded(e) {
    this.setState({
      isExpanded: true,
      isFocused: true
    });
    this._backingSelect.focus();
  },

  handleInertSelectClosed: function handleInertSelectClosed(e) {
    this.setState({
      isExpanded: false,
      hoverIndex: undefined,
      activeIndex: undefined,
      isFocused: true
    });
  },

  handleInertHoverIndex: function handleInertHoverIndex(i, v, e) {
    this.props.onOptionHover(e, v);
    this.setState({
      hoverIndex: i
    });
  },

  handleInertActiveIndex: function handleInertActiveIndex(i, v, e) {
    this.setState({
      activeIndex: i
    });
    this.props.onOptionActive(e, v);
  },

  handleInertSelectIndex: function handleInertSelectIndex(i, v, e) {
    this.props.onChange(e, v);
    this.setState({
      isExpanded: false,
      activeIndex: undefined,
      hoverIndex: undefined
    });
  }
};
module.exports = exports["default"];